import random
import string
from datetime import timedelta

from vk_api.bot_longpoll import VkBotLongPoll, VkBotEventType
from yoomoney import Client, Quickpay

from qiwip2py import QiwiP2P
from src.functions import *
from src.settings import GROUP_ID
from src.steps import *
from src.vk_functions import vk, msg_keyboard, msg, msg_edit

users = {}

if YOOMONEY_TOKEN:
    client = Client(YOOMONEY_TOKEN)
    user = client.account_info()

    print("Account number:", user.account)
    print("Account balance:", user.balance)
    print("Account currency code in ISO 4217 format:", user.currency)
    print("Account status:", user.account_status)
    print("Account type:", user.account_type)
    print("Extended balance information:")
    for pair in vars(user.balance_details):
        print("\t-->", pair, ":", vars(user.balance_details).get(pair))
    print("Information about linked bank cards:")

    cards = user.cards_linked
    if len(cards) != 0:
        for card in cards:
            print(card.pan_fragment, " - ", card.type)
    else:
        print("No card is linked to the account")

if QIWI_SECRET_KEY:
    qiwi_p2p = QiwiP2P(secret_key=QIWI_SECRET_KEY)


def check_attachments(msg):
    try:
        for attachments in msg['message']['attachments']:
            try:
                attachments = attachments[0]
            except:
                pass
            if attachments:
                print('Attachments:', f"{attachments['type']}{attachments[attachments['type']]['owner_id']}_"
                                      f"{attachments[attachments['type']]['id']}_{attachments[attachments['type']]['access_key']}")
    except Exception as E:
        print(E)


def send_by_step(person_id, step):
    attachment = ''
    if STEPS[step].get('attachments'):
        attachment += STEPS[step].get('attachments')
    msg_keyboard(STEPS[step]['text'], person_id,
                 STEPS[step]['keyboard'], attachment=attachment)


def send_by_item_step(person_id, step):
    try:
        this_user_steps = bot_steps[users[person_id]['category']]['category_items'][users[person_id]['item']][
            'item_steps']
        attachment = ''
        if this_user_steps[step].get('attachments'):
            attachment += this_user_steps[step].get('attachments')
        msg_keyboard(this_user_steps[step]['text'], person_id,
                     this_user_steps[step]['keyboard'], attachment=attachment)
    except Exception as E:
        print('Ошибка отправки товара', E)
        msg('Товар временно недоступен', person_id)
        users[person_id]['item_step'] = []
        users[person_id]['category'] = None
        users[person_id]['item'] = None


def gen_bill_id():
    return ''.join(random.sample(f"{string.digits}{string.ascii_letters}{'-'}" * 63, 10))


def qiwi_pay(body, person_id):
    try:
        if not QIWI_SECRET_KEY:
            msg('Метод оплаты пока что недоступен', person_id)
            users[person_id]['item_step'].appedn('step_chose')
        if body == 'QIWI':
            price = bot_steps[users[person_id]['category']]['category_items'][users[person_id]['item']]['price_of_item']
            bill = qiwi_p2p.create_bill(bill_id=gen_bill_id(), amount=price,
                                        expiration_datetime=timedelta(minutes=20))
            users[person_id]['bill_id_qiwi'] = bill.bill_id
            msg_keyboard(f'Оплата через QIWI\nСумма: {price}₽\nСсылка для оплаты:\n{bill.pay_url}', person_id,
                         keyboard=create_keyboard([create_button('Оплатить', link=bill.pay_url)],
                                                  [create_button('Проверить', color='primary')],
                                                  [create_button('Отмена', 'negative')], add_buttons=False))
        elif body == 'Проверить':
            bill = qiwi_p2p.get_bill(users[person_id]['bill_id_qiwi'])
            print(bill)
            print(bill.status['value'])
            if bill.status['value'] == 'PAID':
                print(person_id, 'Оплата прошла успешно!')
                users[person_id]['item_step'].append('step6')
                send_by_item_step(person_id, users[person_id]['item_step'][-1])
                return 0
            else:
                msg('Вы не оплатили счет!', person_id)
                return 0
        elif body == 'Отмена':
            qiwi_p2p.reject_bill(users[person_id]['bill_id_qiwi'])
            users[person_id]['item_step'].append('step_chose')
            send_by_item_step(person_id, users[person_id]['item_step'][-1])
        else:
            msg('Не понимаю вас', person_id)
    except Exception as E:
        print('Ошибка обработки оплаты QIWI', E)


def yoomoney_pay(body, person_id):
    try:
        if not YOOMONEY_TOKEN:
            msg('Метод оплаты пока что недоступен', person_id)
            users[person_id]['item_step'].append('step_chose')
            return 0
        if body == 'ЮКасса':
            price = bot_steps[users[person_id]['category']]['category_items'][users[person_id]['item']]['price_of_item']
            bill_id = gen_bill_id()
            quickpay = Quickpay(
                receiver=user.account,
                quickpay_form="shop",
                targets="Оплата инструкции",
                paymentType="SB",
                sum=price,
                label=bill_id
            )
            print(quickpay)
            print(quickpay.base_url)
            print(quickpay.redirected_url)
            users[person_id]['bill_id_yoomoney'] = bill_id
            msg_keyboard(f'Оплата через ЮКасса\nСумма: {price}₽\nСсылка для оплаты:\n{quickpay.base_url}', person_id,
                         keyboard=create_keyboard([create_button('Оплатить', link=quickpay.base_url)],
                                                  [create_button('Проверить', color='primary')],
                                                  [create_button('Отмена', 'negative')], add_buttons=False))
        elif body == 'Проверить':
            history = client.operation_history(label=users[person_id]['bill_id_yoomoney'])
            print(history.operations)
            for operation in history.operations:
                print("Operation:", operation.operation_id)
                print("\tStatus     -->", operation.status)
                print("\tDatetime   -->", operation.datetime)
                print("\tTitle      -->", operation.title)
                print("\tPattern id -->", operation.pattern_id)
                print("\tDirection  -->", operation.direction)
                print("\tAmount     -->", operation.amount)
                print("\tLabel      -->", operation.label)
                print("\tType       -->", operation.type)
                if operation.status == 'success' and operation.label == users[person_id]['bill_id_yoomoney']:
                    print(person_id, 'Оплата прошла успешно!')
                    users[person_id]['item_step'].append('step6')
                    send_by_item_step(person_id, users[person_id]['item_step'][-1])
                    return 0
                else:
                    msg('Вы не оплатили счет!', person_id)
                    return 0
            msg('Вы не оплатили счет!', person_id)
        elif body == 'Отмена':
            users[person_id]['item_step'].append('step_chose')
            send_by_item_step(person_id, users[person_id]['item_step'][-1])
        else:
            msg('Не понимаю вас', person_id)
    except Exception as E:
        print('Ошибка обработки оплаты YooMoney', E)


def check_user_step(user_id, body):
    try:
        if user_id not in users:
            users[user_id] = {}
            users[user_id]['step'] = ['start']
            print(f"{user_id}:", body, users.get(user_id))
            send_by_step(user_id, 'start')
            return 0
        if users[user_id].get('step') and users[user_id]['step']:
            for el in users[user_id]['step']:
                if users[user_id]['step'].count(el) > 1:
                    users[user_id]['step'].remove(el)
        if users[user_id].get('item_step') and users[user_id]['item_step']:
            for el in users[user_id]['item_step']:
                if users[user_id]['item_step'].count(el) > 1:
                    users[user_id]['item_step'].remove(el)
        print(f"{user_id}:", body, users.get(user_id))
        user_step = users[user_id]['step'][-1]
        user_item_step = users[user_id].get('item_step')
        this_item_steps = None
        if user_item_step and users[user_id].get('category') in bot_steps and users[user_id].get('item') in \
                bot_steps[users[user_id]['category']]['category_items']:
            this_item_steps = bot_steps[users[user_id]['category']]['category_items'][users[user_id]['item']][
                'item_steps']
        if user_item_step and ('step_buy' in user_item_step[-1] or user_item_step[-1] == 'step_chose'):
            print('Покупка')
            new_step = None
            if this_item_steps and user_item_step[-1] in this_item_steps and body in \
                    this_item_steps[user_item_step[-1]]['answers']:
                new_step = this_item_steps[user_item_step[-1]]['answers'][body]
            print(new_step)
            if new_step:
                if new_step == 'step_back':
                    users[user_id]['item_step'].pop(-1)
                    new_step = users[user_id]['item_step'][-1]
                    send_by_item_step(user_id, new_step)
                    return 1
                elif new_step == 'step5':
                    users[user_id]['item_step'].append('step5')
                    send_by_item_step(user_id, 'step5')
                    return 1
                users[user_id]['item_step'].append(new_step)
            if 'qiwi' in users[user_id]['item_step'][-1]:
                qiwi_pay(body, user_id)
            elif 'yoomoney' in users[user_id]['item_step'][-1]:
                yoomoney_pay(body, user_id)
            elif 'vk_pay' in users[user_id]['item_step'][-1]:
                pass
        elif this_item_steps and this_item_steps[user_item_step[-1]].get('answers') and body in \
                this_item_steps[user_item_step[-1]]['answers']:
            new_step = this_item_steps[user_item_step[-1]]['answers'][body]
            if new_step == 'step_back':
                users[user_id]['item_step'].pop(-1)
                new_step = users[user_id]['item_step'][-1]
            if new_step == 'step_buy':
                users[user_id]['item_step'].append(new_step)
                msg('Оплата', user_id)
                return 0
            send_by_item_step(user_id, new_step)
            users[user_id]['item_step'].append(new_step)
        elif user_step in STEPS and STEPS[user_step]['answers'] and body in STEPS[user_step]['answers']:
            new_step = STEPS[user_step]['answers'][body]
            if new_step == 'step_back':
                users[user_id]['step'].pop(-1)
                new_step = users[user_id]['step'][-1]
                if user_item_step:
                    send_by_item_step(user_id, user_item_step[-1])
                else:
                    send_by_step(user_id, new_step)
                users[user_id]['step'].append(new_step)
            else:
                if new_step == 'open_categories' and user_item_step:
                    users[user_id]['item_step'] = None
                    users[user_id]['category'] = None
                    users[user_id]['item'] = None
                    msg_keyboard('Меню', user_id, keyboard=create_keyboard(*[], inline=False, add_buttons=True))
                if new_step == 'actions':
                    users[user_id]['item_step'] = ['step0']
                    category = list(bot_steps.keys())[-1]
                    users[user_id]['category'] = category
                    if user_item_step:
                        msg_keyboard('Меню', user_id, keyboard=create_keyboard(*[], inline=False, add_buttons=True))
                    msg_keyboard(bot_steps[category].get('text_of_category'), user_id,
                                 keyboard=create_keyboard_items(category, bot_steps),
                                 attachment=bot_steps[category].get('attachments_of_category'))
                else:
                    send_by_step(user_id, new_step)
                    if new_step not in ['step9', 'step10', 'step12']:
                        users[user_id]['step'].append(new_step)
        else:
            msg('Не понимаю вас', user_id)
    except Exception as E:
        print('Ошибка обработки сообщения', E)


def check_user_payload(event, user_id, payload):
    try:
        if not isinstance(payload, dict):
            return 0
        action = payload.get('action')
        if not action:
            return 0
        if action == 'open_category':
            category = payload.get('category')
            if category not in bot_steps:
                return 0
            try:
                msg_edit(bot_steps[category].get('text_of_category'), user_id,
                         keyboard=create_keyboard_items(category, bot_steps),
                         message_id=event.object.conversation_message_id, group_id=GROUP_ID,
                         attachment=bot_steps[category].get('attachments_of_category'))
            except Exception as E:
                print('Ошибка изменения сообщения', E)
                msg_keyboard(bot_steps[category].get('text_of_category'), user_id,
                             keyboard=create_keyboard_items(category, bot_steps),
                             attachment=bot_steps[category].get('attachments_of_category'))
            vk.method("messages.sendMessageEventAnswer",
                      {"event_id": event.object.event_id, "user_id": event.object.user_id,
                       "peer_id": event.object.peer_id,
                       "event_data": {}, "random_id": random.randint(1, 2147483647)})
        elif action == 'open_item':
            category = payload.get('category')
            if category not in bot_steps:
                return 0
            item = payload.get('item')
            if item not in bot_steps[category]['category_items']:
                return 0
            if user_id not in users:
                users[user_id] = {}
            users[user_id]['category'] = category
            users[user_id]['item'] = item
            users[user_id]['item_step'] = ['step0']
            users[user_id]['step'] = ['start']
            vk.method("messages.sendMessageEventAnswer",
                      {"event_id": event.object.event_id, "user_id": event.object.user_id,
                       "peer_id": event.object.peer_id,
                       "event_data": {}, "random_id": random.randint(1, 2147483647)})
            # send_by_item_step(user_id, 'step0')
            this_user_steps = bot_steps[users[user_id]['category']]['category_items'][users[user_id]['item']][
                'item_steps']
            attachment = ''
            if this_user_steps['step0'].get('attachments'):
                attachment += this_user_steps['step0'].get('attachments')
            try:
                msg_edit(this_user_steps['step0']['text'], user_id,
                         keyboard=this_user_steps['step0']['keyboard'],
                         message_id=event.object.conversation_message_id, group_id=GROUP_ID,
                         attachment=attachment)
            except Exception as E:
                print('Ошибка изменения сообщения', E)
                msg_keyboard(this_user_steps['step0']['text'], user_id,
                         this_user_steps[step]['keyboard'], attachment=attachment)
        elif action == 'open_page_items':
            category = payload.get('category')
            if category not in bot_steps:
                return 0
            page = payload.get('page')
            if page >= 0:
                try:
                    msg_edit(bot_steps[category].get('text_of_category'), user_id,
                             keyboard=create_keyboard_items(category, bot_steps, page),
                             message_id=event.object.conversation_message_id, group_id=GROUP_ID,
                             attachment=bot_steps[category].get('attachments_of_category'))
                except Exception as E:
                    print('Ошибка изменения сообщения', E)
                    msg_keyboard(bot_steps[category].get('text_of_category'), user_id,
                                 keyboard=create_keyboard_items(category, bot_steps, page),
                                 attachment=bot_steps[category].get('attachments_of_category'))
            else:
                users[user_id]['item_step'] = None
                users[user_id]['category'] = None
                users[user_id]['item'] = None
                attachment = ''
                _step = 'open_categories'
                if STEPS[_step].get('attachments'):
                    attachment += STEPS[_step].get('attachments')
                try:
                    msg_edit(STEPS[_step]['text'], user_id,
                             keyboard=STEPS[_step]['keyboard'],
                             message_id=event.object.conversation_message_id, group_id=GROUP_ID,
                             attachment=attachment)
                except Exception as E:
                    print('Ошибка изменения сообщения', E)
                    msg_keyboard(STEPS[_step]['text'], user_id,
                                 STEPS[_step]['keyboard'], attachment=attachment)

    except Exception as E:
        print('Ошибка обработки payload', E)


def main():
    longpoll = VkBotLongPoll(vk, group_id=GROUP_ID)
    for event in longpoll.listen():
        try:
            # print(event)
            if event.type != VkBotEventType.MESSAGE_EVENT and (
                    event.type != VkBotEventType.MESSAGE_NEW or not event.from_user):
                continue

            if event.type == VkBotEventType.MESSAGE_EVENT:
                _payload = event.object.payload
                person_id = event.obj.user_id
                print(f"{person_id}:", _payload, users.get(person_id))
                check_user_payload(event, person_id, _payload)
                continue
            check_attachments(event.object)
            person_id = event.obj.message['from_id']
            body = event.obj.message['text']
            check_user_step(person_id, body)
        except Exception as longpoll_error:
            print('Ошибка longpoll', longpoll_error)


if __name__ == "__main__":
    print('Бот запущен!')
    while True:
        try:
            main()
        except Exception as E:
            print('Ошибка longpoll', E)
