import json
import random

import requests
from vk_api import vk_api

from .settings import VK_TOKEN

vk = vk_api.VkApi(token=VK_TOKEN)
vk_ = vk.get_api()


def create_button(text, color='secondary', payload=None, link=None):
    if payload:
        return {'color': color, 'action': {'type': "callback", 'payload': payload, 'label': text}}
    if link:
        return {"action": {"type": "open_link", "label": text, 'link': link}}
    return {"action": {"type": "text", "label": text}, "color": color}


def create_keyboard(*buttons, inline=False, add_buttons=True, add_button_download_free=True):
    buttons = list(buttons)
    if add_buttons:
        a = [create_button('Список категорий', color="primary")]
        b = []
        if add_button_download_free:
            b.append(create_button('Скачать бесплатно'))
            b.append(create_button('Акции'))
        else:
            a.append(create_button('Акции'))
        buttons.append(a)
        if b:
            buttons.append(b)
        buttons.append([create_button('Контакты', color='positive'), create_button('Вопрос-Ответ', color='positive')])
    keyboard_buttons = {"one_time": False, 'inline': inline, "buttons": buttons}
    return json.dumps(keyboard_buttons)


def msg_keyboard(text, peer_id, keyboard, attachment=None):
    vk.method("messages.send",
              {"peer_id": peer_id, "message": text, "attachment": attachment, "keyboard": keyboard,
               "random_id": random.randint(1, 2147483647)})


def msg(text, peer_id, attachment=None):
    vk.method("messages.send", {"peer_id": peer_id, "attachment": attachment, "message": text,
                                "random_id": random.randint(1, 2147483647)})


def msg_edit(text, peer_id, keyboard, message_id, group_id, attachment=None):
    vk.method("messages.edit", {"peer_id": peer_id, "attachment": attachment, "message": text, "keyboard": keyboard,
                                "conversation_message_id": message_id, 'group_id': group_id,
                                "random_id": random.randint(1, 2147483647)})


def upload_vk_doc(file_path, user_id, title='Инструкция'):
    upload_url = vk_.docs.getMessagesUploadServer(type='doc', peer_id=user_id)['upload_url']
    response = requests.post(upload_url, files={'file': open(file_path, 'rb')})
    result = json.loads(response.text)
    file = result['file']
    file_info = vk_.docs.save(file=file, title=title, tags=[])
    owner_id = file_info['doc']['owner_id']
    photo_id = file_info['doc']['id']
    return f"doc{owner_id}_{photo_id}"
